#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
using VVVV.Utils.Reflection;
#endregion usings

namespace VVVV.Nodes
{
	public class ClassOne
	{
		public string Test()
		{
			return "Class 1";			
		}
	}
	
	public class ClassTwo
	{
		public int Test()
		{
			return 2;		
		}
	}
	
	public class ClassThree
	{
		public float Test()
		{
			return 0.542f;		
		}
	}
	
	[PluginInfo(Name = "ClassOne", Category = "Test")]
	public class ClassOneNode : IPluginEvaluate
	{
		private INodeOut nodeOutput;
		
		private ClassOne co = new ClassOne();
		
		[ImportingConstructorAttribute()]
		public ClassOneNode(IPluginHost host)
		{
			host.CreateNodeOutput("Output", TSliceMode.Single, TPinVisibility.True, out nodeOutput);
			nodeOutput.SetSubType2(typeof(ClassOne),new Guid[0],"Class One");
			nodeOutput.SetInterface(co);
		}

		public void Evaluate(int SpreadMax) {}
	}
	
	[PluginInfo(Name = "ClassTwo", Category = "Test")]
	public class ClassTwoNode : IPluginEvaluate
	{
		private INodeOut nodeOutput;
		
		private ClassTwo co = new ClassTwo();
		
		[ImportingConstructorAttribute()]
		public ClassTwoNode(IPluginHost host)
		{
			host.CreateNodeOutput("Output", TSliceMode.Single, TPinVisibility.True, out nodeOutput);
			nodeOutput.SetSubType2(typeof(ClassTwo),new Guid[0],"Class Two");
			nodeOutput.SetInterface(co);
		}

		public void Evaluate(int SpreadMax) {}
	}
	
	[PluginInfo(Name = "ClassThree", Category = "Test")]
	public class ClassThreeNode : IPluginEvaluate
	{
		private INodeOut nodeOutput;
		
		private ClassThree co = new ClassThree();
		
		[ImportingConstructorAttribute()]
		public ClassThreeNode(IPluginHost host)
		{
			host.CreateNodeOutput("Output", TSliceMode.Single, TPinVisibility.True, out nodeOutput);
			nodeOutput.SetSubType2(typeof(ClassThree),new Guid[0],"Class Three");
			nodeOutput.SetInterface(co);
		}

		public void Evaluate(int SpreadMax) {}
	}
	
	
	#region PluginInfo
	[PluginInfo(Name = "CustomSink", Category = "Test")]
	#endregion PluginInfo
	public class TestMultiHandlerTestNode : IPluginEvaluate
	{
		
		
		private class CustomConnectionHandler : IConnectionHandler
		{
			private readonly ILogger logger;
			
			public CustomConnectionHandler(ILogger logger)
			{
				this.logger = logger;
			}
					
			public bool Accepts(object source, object sink) 
 			{ 
 				return true;
 			}
			
			public string GetFriendlyNameForSink(object sink) 
 		    { 
 		    	return "One or Two";
 		    }
			
			public string GetFriendlyNameForSource(object sink) 
 		    { 
 		    	return "One or Two";
 		    }
		}
		
		private INodeIn nodeInput;
		
		[ImportingConstructorAttribute()]
		public TestMultiHandlerTestNode(IPluginHost host,ILogger logger)
		{
			host.CreateNodeInput("Input", TSliceMode.Single, TPinVisibility.True, out nodeInput);
			nodeInput.SetConnectionHandler(new CustomConnectionHandler(logger),this);
			nodeInput.SetSubType(new Guid[0],"testing");
		}
		
		[Output("Output")]
		public ISpread<string> FOutput;

		public void Evaluate(int SpreadMax)
		{
			if (nodeInput.IsConnected)
			{
				object test;
				nodeInput.GetUpstreamInterface(out test);
				
				if (test is ClassOne)
				{
					ClassOne co = (ClassOne)test;
					FOutput[0] = co.Test();
				}
				else if (test is ClassTwo)
				{
					ClassTwo co = (ClassTwo)test;
					FOutput[0] = co.Test().ToString();
				}
				else if (test is ClassThree)
				{
					ClassThree co = (ClassThree)test;
					FOutput[0] = co.Test().ToString();
				}
				else 
				{
					FOutput[0] = GetDataType(test.GetType()).Name;
				}
			}
		}
		
		
        private static Type GetDataType(Type ioType)
        {
            if (ioType.IsGenericType)
                return ioType.GetGenericArguments()[0];
            return GetDataType(ioType.BaseType);
        }
	}
}
